

set -ex



pip check
pytest -v --doctest-modules --pyargs cytoolz
echo 'cimport cytoolz ; from cytoolz.functoolz cimport memoize' > try_cimport_cytoolz.pyx
cythonize -i try_cimport_cytoolz.pyx
python -c 'import try_cimport_cytoolz'
exit 0
